package de.intevation.flys.artifacts.states;

import java.util.List;

import org.apache.log4j.Logger;

import org.w3c.dom.Element;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.artifactdatabase.ProtocolUtils;

import de.intevation.artifacts.common.utils.XMLUtils;

import de.intevation.flys.model.Gauge;
import de.intevation.flys.model.Range;

import de.intevation.flys.artifacts.FLYSArtifact;


/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class WQAdapted extends DefaultState {

    /** The logger used in this state.*/
    private static Logger logger = Logger.getLogger(WQAdapted.class);


    /**
     * This method creates one element for each gauge of the selected river that
     * is intersected by the given kilometer range. Each element is a tuple of
     * (from;to) where <i>from</i> is the lower bounds of the gauge or the lower
     * kilometer range. <i>to</i> is the upper bounds of the gauge or the upper
     * kilometer range.
     *
     * @param cr The ElementCreator.
     * @param artifact The FLYS artifact.
     * @param name The name of the data item.
     * @param context The CallContext.
     *
     * @return a list of elements that consist of tuples of the intersected
     * gauges of the selected river.
     */
    protected Element[] createItems(
        XMLUtils.ElementCreator cr,
        Artifact    artifact,
        String      name,
        CallContext context)
    {
        logger.debug("WQAdapted.createItems");

        FLYSArtifact flysArtifact = (FLYSArtifact) artifact;

        double[]    dist   = flysArtifact.getDistance();
        List<Gauge> gauges = flysArtifact.getGauges();

        int num = gauges != null ? gauges.size() : 0;

        if (num == 0) {
            logger.warn("Selected distance matches no gauges.");
            return null;
        }

        Element[] elements = new Element[num];

        int idx = 0;

        for (Gauge gauge: gauges) {
            Range range  = gauge.getRange();
            double lower = range.getA().doubleValue();
            double upper = range.getB().doubleValue();

            double from = dist[0] < lower ? lower : dist[0];
            double to   = dist[1] > upper ? upper : dist[1];

            String key = Double.toString(from) + ";" + Double.toString(to);

            elements[idx++] = createItem(cr, new String[] {key, ""});
        }

        return elements;
    }


    protected Element createItem(XMLUtils.ElementCreator cr, Object obj) {
        Element item  = ProtocolUtils.createArtNode(cr, "item", null, null);
        Element label = ProtocolUtils.createArtNode(cr, "label", null, null);
        Element value = ProtocolUtils.createArtNode(cr, "value", null, null);

        String[] arr = (String[]) obj;

        label.setTextContent(arr[0]);
        value.setTextContent(arr[1]);

        item.appendChild(label);
        item.appendChild(value);

        return item;
    }


    @Override
    protected String getUIProvider() {
        return "wq_panel_adapted";
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
